@@ -25,12 +25,12 @@ class Mamba2Block(nn.Module):
         d_in_proj = 2 * self.d_inner + 2 * self.d_state + self.n_heads
         self.in_proj = nn.Linear(self.d_model, d_in_proj, bias=False, device=device)
 
-        conv_dim = self.d_inner + 2 * self.d_state
+        cond_inner = self.d_inner + 2 * self.d_state
         self.conv1d = nn.Conv1d(
-            in_channels=conv_dim,
-            out_channels=conv_dim,
+            in_channels=cond_inner,
+            out_channels=cond_inner,
             kernel_size=self.d_conv,
-            groups=conv_dim,
+            groups=cond_inner,
             padding=self.d_conv-1,
             device=device,
         )
@@ -279,19 +279,18 @@ class SeqLinear(nn.Module):
     Linear layer with weights derived from a weighted sum of rank-1 sequential updates.
     """
     def __init__(self, d_model, n_heads,
-                 W_0=False, d_conv=4,
-                 u_dim=None, v_dim=None,
+                 d_conv=4, d_state=None, d_inner=None,
                  device="cpu"):
         super().__init__()
         self.d_model = d_model
-        self.u_dim = u_dim if u_dim is not None else d_model
-        self.v_dim = v_dim if v_dim is not None else d_model
+        self.d_state = d_state if d_state is not None else d_model
+        self.d_inner = d_inner if d_inner is not None else d_model
         self.n_heads = n_heads
         self.device = device
         
-        d_in_proj = 2 * self.u_dim + self.v_dim + self.n_heads
+        d_in_proj = 2 * self.d_state + self.d_inner + self.n_heads
         self.in_proj = nn.Linear(d_model, d_in_proj, bias=False, device=device)
-        d_conv_c = 2 * self.u_dim + self.v_dim
+        d_conv_c = 2 * self.d_state + self.d_inner
         self.conv = nn.Conv1d(
             in_channels=d_conv_c,
             out_channels=d_conv_c,
@@ -301,8 +300,7 @@ class SeqLinear(nn.Module):
             device=device,
         )
         self.dt_bias = nn.Parameter(torch.empty(n_heads, device=device))
-        self.W_0 = nn.Parameter(torch.empty(n_heads, self.u_dim // n_heads, self.v_dim // n_heads, device=device)) if W_0 else None
-        self.out_proj = nn.Linear(self.v_dim, d_model, bias=False, device=device)
+        self.out_proj = nn.Linear(self.d_inner, d_model, bias=False, device=device)
         
         self._reset_parameters()
     
@@ -318,9 +316,7 @@ class SeqLinear(nn.Module):
             nn.init.constant_(self.in_proj.bias, 0.)
         if self.out_proj.bias is not None:
             nn.init.constant_(self.out_proj.bias, 0.)
-        if self.W_0 is not None:
-            nn.init.constant_(self.W_0, 0.)
-
+    
     def bidirectional(self, x):
         bsz, src_len, d_model = x.size()
         
@@ -329,7 +325,7 @@ class SeqLinear(nn.Module):
         x, dt = torch.split(
             x,
             [
-                2 * self.u_dim + self.v_dim,
+                2 * self.d_state + self.d_inner,
                 self.n_heads,
             ],
             dim=-1
@@ -352,51 +348,43 @@ class SeqLinear(nn.Module):
             x_f = self.conv(x.transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
             x_b = self.conv(x.flip(1).transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
         
-        z_f, u_f, v_f = torch.split(
+        C_f, B_f, x_f = torch.split(
             x_f,
             [
-                self.u_dim,
-                self.u_dim,
-                self.v_dim,
+                self.d_state,
+                self.d_state,
+                self.d_inner,
             ],
             dim=-1
         )
-        z_b, u_b, v_b = torch.split(
+        C_b, B_b, x_b = torch.split(
             x_b,
             [
-                self.u_dim,
-                self.u_dim,
-                self.v_dim,
+                self.d_state,
+                self.d_state,
+                self.d_inner,
             ],
             dim=-1
         )
-                
-        log_dt_f = torch.cumsum(-F.gelu(dt + self.dt_bias), dim=1)
-        log_dt_f = log_dt_f - log_dt_f.topk(1, dim=1).values
-        dt_f = torch.exp(log_dt_f)
-        z_f = rearrange(z_f, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
-        u_f = rearrange(u_f, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
-        v_f = rearrange(v_f, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         
-        log_dt_b = torch.cumsum(-F.gelu(dt.flip() + self.dt_bias), dim=1)
-        log_dt_b = log_dt_b - log_dt_b.topk(1, dim=1).values
-        dt_b = torch.exp(log_dt_b)
-        z_b = rearrange(z_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
-        u_b = rearrange(u_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
-        v_b = rearrange(v_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
+        dt_f = F.softplus(dt + self.dt_bias)
+        C_f = rearrange(C_f, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
+        B_f = rearrange(B_f, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
+        x_f = rearrange(x_f, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         
-        W_f = torch.cumsum(torch.einsum('bsh, bshd, bshe -> bshde', dt_f, u_f, v_f), dim=1)
-        W_b = torch.cumsum(torch.einsum('bsh, bshd, bshe -> bshde', dt_b, u_b, v_b), dim=1)
+        dt_b = F.softplus(dt.flip(1) + self.dt_bias)
+        C_b = rearrange(C_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
+        B_b = rearrange(B_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
+        x_b = rearrange(x_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         
-        if self.W_0 is not None:
-            W_f = W_f + self.W_0.unsqueeze(1)
-            W_b = W_b + self.W_0.unsqueeze(1)
+        W_f = torch.cumsum(torch.einsum('bsh, bshd, bshe -> bshde', dt_f, B_f, x_f), dim=1)
+        W_b = torch.cumsum(torch.einsum('bsh, bshd, bshe -> bshde', dt_b, B_b, x_b), dim=1)
         
         norm_f = torch.cumsum(dt_f, dim=1).unsqueeze(-1)
         norm_b = torch.cumsum(dt_b, dim=1).unsqueeze(-1)
         
-        out_f = torch.matmul(z_f.unsqueeze(-2), W_f).squeeze(-2) / norm_f
-        out_b = torch.matmul(z_b.unsqueeze(-2), W_b).squeeze(-2) / norm_b
+        out_f = torch.einsum('bshd, bshde -> bshe', C_f, W_f) / norm_f
+        out_b = torch.einsum('bshd, bshde -> bshe', C_b, W_b) / norm_b
         
         out = rearrange(out_f + out_b, 'b s h d -> b s (h d)')
         out = self.out_proj(out)
@@ -411,7 +399,7 @@ class SeqLinear(nn.Module):
         x, dt = torch.split(
             x,
             [
-                2 * self.u_dim + self.v_dim,
+                2 * self.d_state + self.d_inner,
                 self.n_heads,
             ],
             dim=-1
@@ -427,33 +415,30 @@ class SeqLinear(nn.Module):
         else:
             x = self.conv(x.transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
         
-        z, u, v = torch.split(
+        C, B, x = torch.split(
             x,
             [
-                self.u_dim,
-                self.u_dim,
-                self.v_dim,
+                self.d_state,
+                self.d_state,
+                self.d_inner,
             ],
             dim=-1
         )
         
-        log_dt = torch.cumsum(-F.gelu(dt + self.dt_bias), dim=1)
-        log_dt = log_dt - log_dt.topk(1, dim=1).values
-        dt = torch.exp(log_dt)
-        z = rearrange(z, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
-        u = rearrange(u, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
-        v = rearrange(v, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
+        C = rearrange(C, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
+        B = rearrange(B, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
+        x = rearrange(x, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         
         if mode == "causal":
-            norm = torch.cumsum(dt, dim=1).unsqueeze(-1)
-            W = torch.cumsum(torch.einsum('bsh, bshd, bshe -> bshde', dt, u, v), dim=1)
+            dt = F.softplus(dt + self.dt_bias)
+            norm = torch.cumsum(dt, dim=-1).unsqueeze(-1)
+            W = torch.cumsum(torch.einsum('bsh, bshd, bshe -> bshde', dt, C, B), dim=1)
+            out = torch.einsum('bshd, bshde -> bshe', x, W) / norm
         elif mode == "noncausal":
+            dt = F.softplus(dt + self.dt_bias)
             norm = torch.sum(dt, dim=1, keepdim=True).unsqueeze(-1)
-            W = torch.einsum('bsh, bshd, bshe -> bhde', dt, u, v).unsqueeze(1)
+            out = torch.einsum('bshd, bthd, bthe -> bshe', x, C, B) / src_len
         
-        if self.W_0 is not None: W = W + self.W_0.unsqueeze(1)
-        
-        out = torch.einsum('bshd, bshde -> bshe', z, W) / norm
         out = rearrange(out, 'b s h d -> b s (h d)')
         out = self.out_proj(out)
         return out
@@ -461,8 +446,8 @@ class SeqLinear(nn.Module):
 class SeqMLP(nn.Module):
     def __init__(self, emb_dim, input_dim, output_dim,
                  n_layers=1, n_heads=1, dropout=0.0,
-                 u_dim=None, v_dim=None, W_0=False,
-                 d_conv=4, pos_encoding=False, pos_encoding_max_len=None,
+                 d_state=None, d_inner=None, d_conv=4,
+                 pos_encoding=False, pos_encoding_max_len=None,
                  use_embedding=True, weight_tying=False,
                  mode="causal", device="cpu"):
         super().__init__()
@@ -492,9 +477,8 @@ class SeqMLP(nn.Module):
                     mixer=SeqLinear(
                         d_model=emb_dim,
                         n_heads=n_heads,
-                        u_dim=u_dim,
-                        v_dim=v_dim,
-                        W_0=W_0,
+                        d_state=d_state,
+                        d_inner=d_inner,
                         d_conv=d_conv,
                         device=device
                     ),
